@@ -1,4 +1,5 @@
 import os
+import logging
 import cv2
 import numpy as np
 from typing import List, Literal
@@ -7,8 +8,13 @@ from OpenGL.GL import *
 from OpenGL.GLU import *
 from .scene_element import Scene
 
-# Pygameの警告を抑制
+# ---------------------------------------------------------
+# Silence optional initialisation warnings from pygame and PyOpenGL
+# ---------------------------------------------------------
 os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = '1'
+os.environ.setdefault('PYOPENGL_DISABLE_WARNINGS', '1')
+logging.getLogger('OpenGL').setLevel(logging.ERROR)
+logging.getLogger('OpenGL.plugins').setLevel(logging.ERROR)
 import pygame
 
 # オーディオ処理用ライブラリの試行インポート
@@ -19,30 +25,31 @@ try:
 except ImportError:
     HAS_FFMPEG = False
 
+_LIBX264_WARNING_EMITTED = False
 
 def has_audio_stream(video_path: str) -> bool:
     """Check if a video file has an audio stream using ffprobe.
-
+    
     Args:
         video_path: Path to the video file
-
+        
     Returns:
         True if the video has at least one audio stream, False otherwise
     """
     if not HAS_FFMPEG:
         return False
-
+    
     if not os.path.exists(video_path):
         return False
-
+    
     try:
         # Use ffprobe to check for audio streams
         cmd = [
-            'ffprobe', '-v', 'quiet', '-print_format', 'json',
+            'ffprobe', '-v', 'quiet', '-print_format', 'json', 
             '-show_streams', '-select_streams', 'a', video_path
         ]
         result = subprocess.run(cmd, capture_output=True, text=True, timeout=10)
-
+        
         if result.returncode == 0:
             try:
                 data = json.loads(result.stdout)
@@ -51,7 +58,7 @@ def has_audio_stream(video_path: str) -> bool:
             except json.JSONDecodeError:
                 return False
         return False
-
+        
     except (subprocess.CalledProcessError, subprocess.TimeoutExpired, FileNotFoundError):
         # Fallback: try to use OpenCV to detect audio properties
         try:
@@ -64,95 +71,13 @@ def has_audio_stream(video_path: str) -> bool:
             cap.release()
         except:
             pass
-
+        
         return False
 
 
-def detect_nvenc_support() -> bool:
-    """Detect if NVIDIA NVENC hardware encoding is available.
-
-    Returns:
-        True if NVENC is available, False otherwise
-    """
-    if not HAS_FFMPEG:
-        return False
-
-    try:
-        # Check if ffmpeg has h264_nvenc encoder
-        cmd = ['ffmpeg', '-hide_banner', '-encoders']
-        result = subprocess.run(cmd, capture_output=True, text=True, timeout=5)
-
-        if result.returncode == 0:
-            # Look for h264_nvenc in the output
-            return 'h264_nvenc' in result.stdout
-        return False
-
-    except (subprocess.CalledProcessError, subprocess.TimeoutExpired, FileNotFoundError):
-        return False
-
-
-class FFmpegWriter:
-    """Wrapper class for FFmpeg process that mimics cv2.VideoWriter interface."""
-
-    def __init__(self, process: subprocess.Popen, width: int, height: int):
-        """Initialize FFmpeg writer.
-
-        Args:
-            process: FFmpeg subprocess
-            width: Video width
-            height: Video height
-        """
-        self.process = process
-        self.width = width
-        self.height = height
-        self._is_opened = True
-
-    def isOpened(self) -> bool:
-        """Check if the writer is opened.
-
-        Returns:
-            True if writer is ready to accept frames
-        """
-        return self._is_opened and self.process.poll() is None
-
-    def write(self, frame: np.ndarray) -> None:
-        """Write a frame to the video.
-
-        Args:
-            frame: BGR image array (numpy array)
-        """
-        if not self.isOpened():
-            raise Exception("FFmpeg writer is not opened")
-
-        try:
-            # Write frame to FFmpeg stdin
-            self.process.stdin.write(frame.tobytes())
-        except BrokenPipeError:
-            # FFmpeg process has terminated
-            self._is_opened = False
-            raise Exception("FFmpeg process terminated unexpectedly")
-
-    def release(self) -> None:
-        """Release the writer and close FFmpeg process."""
-        if self.process.stdin:
-            self.process.stdin.close()
-
-        # Wait for FFmpeg to finish encoding
-        self.process.wait()
-
-        self._is_opened = False
-
-        # Check if there were any errors
-        if self.process.returncode != 0:
-            stderr_output = self.process.stderr.read().decode('utf-8') if self.process.stderr else ""
-            print(f"Warning: FFmpeg encoding finished with return code {self.process.returncode}")
-            if stderr_output:
-                print(f"FFmpeg stderr: {stderr_output}")
-
-
 class MasterScene:
     """マスターシーンクラス - 全体の動画を管理"""
-    def __init__(self, output_filename: str = "output_video.mp4", width: int = 1920, height: int = 1080, fps: int = 60, quality: Literal["low", "medium", "high"] = "medium", use_gpu_encoding: bool = True, use_egl: bool = True):
+    def __init__(self, output_filename: str = "output_video.mp4", width: int = 1920, height: int = 1080, fps: int = 60, quality: Literal["low", "medium", "high"] = "medium"):
         self.width = width
         self.height = height
         self.fps = fps
@@ -162,14 +87,7 @@ class MasterScene:
         self.output_filename = output_filename
         self.audio_elements = []  # オーディオ要素を追跡
         self._has_content_at_start = False  # Track if master scene has content at time 0
-
-        # GPU encoding settings
-        self.use_gpu_encoding = use_gpu_encoding
-        self._nvenc_available = None  # Cache for NVENC detection
-
-        # Headless rendering settings (Linux only)
-        self.use_egl = use_egl  # Try EGL for headless rendering on Linux
-
+        
         # 品質設定に基づいてスーパーサンプリング倍率を設定
         self.quality_multipliers = {
             "low": 1,     # 1x (現在の動作)
@@ -358,106 +276,75 @@ class MasterScene:
         glEnable(GL_BLEND)
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
     
-    def _is_nvenc_available(self) -> bool:
-        """Check if NVENC is available (with caching).
-
-        Returns:
-            True if NVENC is available and should be used
-        """
-        if self._nvenc_available is None:
-            self._nvenc_available = self.use_gpu_encoding and detect_nvenc_support()
-            if self._nvenc_available:
-                print("✓ NVIDIA NVENC hardware encoding detected - using GPU acceleration")
-            elif self.use_gpu_encoding:
-                print("✗ NVIDIA NVENC not available - falling back to CPU encoding")
-        return self._nvenc_available
-
     def _setup_video_writer(self):
-        """動画書き込み設定"""
-        # オーディオ要素がある場合は一時ファイル、ない場合は指定ファイル名
+        # ---------------------------------------------------------
+        # Launch an FFmpeg/libx264 encoder that accepts raw BGR frames
+        # ---------------------------------------------------------
         if self.audio_elements:
-            # 一時ファイル名を作成
             base_name = os.path.splitext(self.output_filename)[0]
             ext = os.path.splitext(self.output_filename)[1]
             full_path = f"{base_name}_temp_video_only{ext}"
         else:
             full_path = self.output_filename
-
-        # Check if we should use GPU encoding
-        use_nvenc = self._is_nvenc_available()
-
-        if use_nvenc:
-            # Use FFmpeg with NVENC for GPU encoding
-            return self._setup_ffmpeg_writer(full_path), full_path
-        else:
-            # Use OpenCV VideoWriter for CPU encoding (fallback)
-            fourcc = cv2.VideoWriter_fourcc(*'mp4v')
-            video_writer = cv2.VideoWriter(full_path, fourcc, self.fps, (self.width, self.height))
-
-            if not video_writer.isOpened():
-                raise Exception(f"動画ファイル {full_path} を作成できませんでした")
-
-            return video_writer, full_path
-
-    def _setup_ffmpeg_writer(self, output_path: str):
-        """Setup FFmpeg writer with NVENC encoding.
-
-        Args:
-            output_path: Path to output video file
-
-        Returns:
-            FFmpeg process wrapper object
-        """
+        
         if not HAS_FFMPEG:
-            raise Exception("FFmpeg is required for GPU encoding")
-
-        # FFmpeg command for NVENC encoding
-        # -f rawvideo: input format is raw video
-        # -pix_fmt rgb24: pixel format
-        # -s WxH: video size
-        # -r fps: frame rate
-        # -i -: read from stdin
-        # -c:v h264_nvenc: use NVENC encoder
-        # -preset fast: encoding preset (fast, medium, slow)
-        # -b:v: bitrate (adjust based on quality)
-        # -pix_fmt yuv420p: output pixel format for compatibility
-
-        # Determine bitrate based on quality
-        bitrate_map = {
-            "low": "5M",
-            "medium": "10M",
-            "high": "20M"
-        }
-        bitrate = bitrate_map.get(self.quality, "10M")
-
-        cmd = [
+            raise RuntimeError("FFmpeg with libx264 support is required to encode browser-compatible video.")
+        
+        ffmpeg_cmd = [
             'ffmpeg',
-            '-y',  # Overwrite output file
+            '-y',
             '-f', 'rawvideo',
-            '-vcodec', 'rawvideo',
-            '-pix_fmt', 'bgr24',  # OpenCV uses BGR
+            '-pix_fmt', 'bgr24',
             '-s', f'{self.width}x{self.height}',
             '-r', str(self.fps),
-            '-i', '-',  # Read from stdin
-            '-c:v', 'h264_nvenc',
-            '-preset', 'fast',
-            '-b:v', bitrate,
+            '-i', '-',
+            '-an',
+            '-c:v', 'libx264',
+            '-preset', 'medium',
+            '-profile:v', 'baseline',
+            '-level', '3.1',
             '-pix_fmt', 'yuv420p',
-            output_path
+            '-movflags', '+faststart',
+            '-g', str(self.fps),
+            '-keyint_min', str(self.fps),
+            full_path
         ]
-
-        # Start FFmpeg process
+        
         try:
             process = subprocess.Popen(
-                cmd,
+                ffmpeg_cmd,
                 stdin=subprocess.PIPE,
-                stdout=subprocess.PIPE,
+                stdout=subprocess.DEVNULL,
                 stderr=subprocess.PIPE
             )
-            # Return a wrapper object that mimics cv2.VideoWriter interface
-            return FFmpegWriter(process, self.width, self.height)
-        except FileNotFoundError:
-            raise Exception("FFmpeg not found. Please install FFmpeg to use GPU encoding.")
+        except FileNotFoundError as exc:
+            raise RuntimeError("FFmpeg is not installed or not reachable on PATH. Install FFmpeg with libx264 support.") from exc
+        
+        return process, full_path, ffmpeg_cmd
+
+    def _finalize_video_writer(self, process, ffmpeg_cmd):
+        # ---------------------------------------------------------
+        # Close the encoder stdin and surface any FFmpeg/libx264 errors
+        # ---------------------------------------------------------
+        if process.stdin:
+            try:
+                process.stdin.close()
+            except BrokenPipeError:
+                pass
+            finally:
+                process.stdin = None
+        
+        _, stderr_data = process.communicate()
+        if process.returncode != 0:
+            error_text = stderr_data.decode('utf-8', errors='replace') if stderr_data else ""
+            global _LIBX264_WARNING_EMITTED
+            if "Unknown encoder 'libx264'" in error_text and not _LIBX264_WARNING_EMITTED:
+                print("Warning: FFmpeg was built without libx264. Install FFmpeg with x264 support to render Chrome-compatible video.")
+                _LIBX264_WARNING_EMITTED = True
+            raise RuntimeError(f"FFmpeg command failed ({' '.join(ffmpeg_cmd)}): {error_text}")
+        
+        if process.stderr:
+            process.stderr.close()
     
     def _capture_frame(self):
         """現在の画面をキャプチャ"""
@@ -675,21 +562,7 @@ class MasterScene:
                 os.environ['SDL_VIDEODRIVER'] = 'cocoa'
             elif system == 'Windows':
                 os.environ['SDL_VIDEODRIVER'] = 'windows'
-            elif system == 'Linux':
-                # LinuxでNVIDIA GPUを優先的に使用
-                if 'LIBGL_ALWAYS_SOFTWARE' not in os.environ:
-                    os.environ['LIBGL_ALWAYS_SOFTWARE'] = '0'  # ソフトウェアレンダリングを無効化
-
-                # NVIDIA GPU使用を強制（利用可能な場合）
-                if '__GLX_VENDOR_LIBRARY_NAME' not in os.environ:
-                    # nvidia-smiが利用可能ならNVIDIA GPUが存在
-                    try:
-                        subprocess.run(['nvidia-smi'], capture_output=True, check=True, timeout=2)
-                        os.environ['__GLX_VENDOR_LIBRARY_NAME'] = 'nvidia'
-                        os.environ['__NV_PRIME_RENDER_OFFLOAD'] = '1'
-                        os.environ['__VK_LAYER_NV_optimus'] = 'NVIDIA_only'
-                    except (subprocess.CalledProcessError, FileNotFoundError, subprocess.TimeoutExpired):
-                        pass  # NVIDIA GPUがない場合はデフォルト設定
+            # Linuxの場合は設定しない（X11/Xvfbを自動選択）
 
             # コンテナ環境または明示的にDISPLAYがない場合は音声をdummyに
             if is_container or not os.environ.get('DISPLAY'):
@@ -698,80 +571,36 @@ class MasterScene:
         os.environ['SDL_VIDEO_WINDOW_POS'] = '-1000,-1000'
         os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = '1'
 
-        # Try headless context first (Linux only)
-        headless_context = None
-        context_type = None
+        # Pygameを初期化
+        pygame.init()
 
-        if platform.system() == 'Linux' and self.use_egl:
-            try:
-                from .headless_context import HeadlessContext
-                headless_context = HeadlessContext(self.render_width, self.render_height, use_egl=True)
-                context_type, _ = headless_context.create()
-                print(f"Using {context_type.upper()} for rendering")
-            except Exception as e:
-                print(f"Headless context creation failed: {e}")
-                print("Falling back to Pygame...")
-                headless_context = None
-
-        # Fallback to Pygame if headless context failed or not on Linux
-        if headless_context is None:
-            # Pygameを初期化
-            pygame.init()
-
-            # OpenGLウィンドウを作成（高解像度レンダリング用）
-            # ヘッドレス環境でも動作するようにフラグを調整
-            try:
-                screen = pygame.display.set_mode(
-                    (self.render_width, self.render_height),
-                    pygame.DOUBLEBUF | pygame.OPENGL | pygame.HIDDEN
-                )
-                context_type = "pygame"
-            except pygame.error as e:
-                # OpenGLが使えない場合は通常のサーフェスモードで試行
-                print(f"Warning: OpenGL mode failed ({e}), trying software rendering...")
-                screen = pygame.display.set_mode(
-                    (self.render_width, self.render_height),
-                    pygame.HIDDEN
-                )
-                # ソフトウェアレンダリングモードではOpenGLは使用できないため、
-                # 代替レンダリングパスが必要
-                raise NotImplementedError("Software rendering mode is not yet implemented. OpenGL is required.")
+        # OpenGLウィンドウを作成（高解像度レンダリング用）
+        # ヘッドレス環境でも動作するようにフラグを調整
+        try:
+            screen = pygame.display.set_mode(
+                (self.render_width, self.render_height),
+                pygame.DOUBLEBUF | pygame.OPENGL | pygame.HIDDEN
+            )
+        except pygame.error as e:
+            # OpenGLが使えない場合は通常のサーフェスモードで試行
+            print(f"Warning: OpenGL mode failed ({e}), trying software rendering...")
+            screen = pygame.display.set_mode(
+                (self.render_width, self.render_height),
+                pygame.HIDDEN
+            )
+            # ソフトウェアレンダリングモードではOpenGLは使用できないため、
+            # 代替レンダリングパスが必要
+            raise NotImplementedError("Software rendering mode is not yet implemented. OpenGL is required.")
         
         # OpenGLを初期化
         self._init_opengl()
-
-        # OpenGL情報を表示（GPU利用状況の確認）
-        try:
-            vendor = glGetString(GL_VENDOR)
-            renderer = glGetString(GL_RENDERER)
-            gl_version = glGetString(GL_VERSION)
-
-            vendor_str = vendor.decode('utf-8') if vendor else 'Unknown'
-            renderer_str = renderer.decode('utf-8') if renderer else 'Unknown'
-            version_str = gl_version.decode('utf-8') if gl_version else 'Unknown'
-
-            print(f"OpenGL Renderer: {renderer_str}")
-
-            # ソフトウェアレンダリングの警告
-            if 'llvmpipe' in renderer_str.lower() or 'softpipe' in renderer_str.lower():
-                print("⚠️  WARNING: Software rendering detected (Mesa llvmpipe)")
-                print("   → GPU acceleration is NOT being used for OpenGL rendering")
-                print("   → Performance will be significantly degraded")
-                print("   → Check NVIDIA driver installation: nvidia-smi")
-                print("   → Set environment variables: __GLX_VENDOR_LIBRARY_NAME=nvidia")
-        except Exception as e:
-            print(f"Could not retrieve OpenGL info: {e}")
-
+        
         # 動画書き込み設定
-        video_writer, video_path = self._setup_video_writer()
-
-        # Show encoding information
-        encoding_method = "GPU (NVENC)" if self._is_nvenc_available() else "CPU"
-        print(f"Encoding: {encoding_method} | Resolution: {self.width}x{self.height} | FPS: {self.fps} | Quality: {self.quality}")
-
+        video_process, video_path, ffmpeg_cmd = self._setup_video_writer()
+        
         try:
             total_frames = int(self.total_duration * self.fps)
-
+            
             # 品質設定をすべてのシーンに適用（一度だけ）
             for scene in self.scenes:
                 self._apply_quality_to_scene(scene)
@@ -787,31 +616,27 @@ class MasterScene:
                     # 全シーンをレンダリング
                     for scene in self.scenes:
                         scene.render(current_time)
-
-                    # 描画を確定 (Pygame使用時のみ)
-                    if context_type == "pygame":
-                        pygame.display.flip()
-                    elif context_type == "egl":
-                        # EGLの場合はglFlushで十分
-                        glFlush()
-
+                    
+                    # 描画を確定
+                    pygame.display.flip()
+                    
                     # フレームをキャプチャして動画に書き込み
                     frame = self._capture_frame()
-                    video_writer.write(frame)
+                    try:
+                        video_process.stdin.write(frame.tobytes())
+                    except BrokenPipeError as exc:
+                        raise RuntimeError("FFmpeg encoder pipe closed unexpectedly. Check FFmpeg installation.") from exc
                     
                     # プログレスバーを更新
                     pbar.update(1)
             
         finally:
             # クリーンアップ
-            video_writer.release()
-
-            # Context cleanup
-            if headless_context is not None:
-                headless_context.destroy()
-            else:
+            try:
+                self._finalize_video_writer(video_process, ffmpeg_cmd)
+            finally:
                 pygame.quit()
-
+            
             # オーディオミキシング（ビデオ作成後）
             if self.audio_elements:
-                final_output = self._create_audio_mix(video_path)
\ No newline at end of file
+                final_output = self._create_audio_mix(video_path)
